import { d as MessagesMap } from './messages-4c34bad5.js';
import { L as LocaleDescriptor, a as Locale } from './locales-8cef3d6b.js';

declare const AsyncEventSymbol: unique symbol;
interface AsyncEvent {
    /** Property used to identify async event. */
    [AsyncEventSymbol]: true;
}
declare function markAsAsync(event: AsyncEvent): void;
/**
 * Checks whether the event is async event.
 *
 * @param event Event to check.
 * @returns Whether the event must be executed in async manner.
 */
declare function isAsyncEvent(event: Event): event is Event & AsyncEvent;

/** Represents an event. */
interface Event {
    /** Type of the event. */
    readonly type: string;
}
/** Represent options for adding an event listener. */
interface EventListenerOptions {
    /**
     * Whether the event must be triggered only once.
     *
     * If set to `true`, the event listener will be removed once triggered,
     * regardless whether the triggering call was successful or resulted in an
     * error. The error will still propagate to the event caller.
     *
     * @default false // Can be called multiple times.
     */
    once?: boolean;
    /**
     * The priority of the event listener.
     *
     * The higher the number, the upper in calling queue the listener will be.
     * Therefore if there are multiple event listeners A (`0`), B (`100`), C
     * (`-100`), they will be called in the order from highest priority to lowest:
     * B (`100`), A (`0`), C(`-100`). This is useful if you want to declare
     * monitor events that do not do anything on their own, but record the results
     * of all other events.
     *
     * @default 0 // Has regular priority.
     */
    priority?: number;
    /**
     * Whether the listener is always called, regardless of whether the event is
     * cancelled or not.
     *
     * @default false // Does not listen to cancelled events.
     */
    always?: boolean;
}
type EventTypes<EventsUnion extends Event> = EventsUnion['type'];
type EventObjectFrom<EventsUnion extends Event, EventType extends EventTypes<EventsUnion>> = EventsUnion extends {
    type: EventType;
} ? EventsUnion : never;
/**
 * Represents a generic event target.
 *
 * @template EventsUnion A union of event types that this event triggers by
 *   itself.
 */
interface EventTarget<EventsUnion extends Event, ThisType> {
    /**
     * Adds the event listener for the provided event type or does nothing if it
     * is already added.
     *
     * @param type Type of the event to which listener will listen.
     * @param listener Listener to be called on event.
     * @param options Options for event listener.
     */
    addEventListener<EventType extends EventTypes<EventsUnion>>(type: EventType, listener: (this: ThisType, event: EventObjectFrom<EventsUnion, EventType>) => any, options?: EventListenerOptions): void;
    addEventListener(type: string, listener: (this: ThisType, event: Event) => any, options?: EventListenerOptions): void;
    /**
     * Removes the event listener for the provided event type or does nothing if
     * it was already removed.
     *
     * @param type Type of the event to which the listener is subscribed.
     * @param listener Listener to be removed.
     */
    removeEventListener<EventType extends EventTypes<EventsUnion>>(type: EventType, listener: (this: ThisType, event: EventObjectFrom<EventsUnion, EventType>) => any): void;
    removeEventListener(type: string, listener: (this: ThisType, event: Event) => any): void;
    /**
     * Dispatches an event to all the listeners of its type.
     *
     * @param event Event to be dispatched.
     * @returns `true` if event wasn't cancelled after the propagation or is not
     *   cancellable to begin with, `false` otherwise. If event is asynchronous,
     *   then the value returned will be.
     */
    dispatchEvent<E extends Event>(event: E): E extends AsyncEvent ? Promise<boolean> : boolean;
}

declare const CollectableSymbol: unique symbol;
declare const CollectedPropertyKey = "collected";
declare const CollectPropertyKey = "collect";
/**
 * Represents a collectable event that returns {@link T} on collection.
 *
 * @template T The result of collection.
 */
interface CollectableEvent<T> {
    /** Property used to identify collectable event. */
    [CollectableSymbol]: true;
    /**
     * Whether the data from the event has been collected or not. If event data
     * has been collected, none of the event's values must be used anymore since
     * they will not be used by the collector a second time, as well as may not
     * represent the actual state of the event.
     */
    get [CollectedPropertyKey](): boolean;
    /**
     * Calls collector, returns the collection result and marks the event as
     * collected.
     */
    [CollectPropertyKey](): T;
}
declare function implementCollection<R>(event: CollectableEvent<R>, collector: (this: void) => R): void;

declare const Canceled: unique symbol;
declare const CanceledPropertyKey = "canceled";
declare const CancelPropertyKey = "cancel";
interface CancelableEvent {
    [Canceled]: true;
    /** Read-only value representing whether the event has been cancelled. */
    get [CanceledPropertyKey](): boolean;
    /** Cancels event. Once cancelled, event cannot be un-cancelled. */
    [CancelPropertyKey](): void;
}
/**
 * Implements {@link CancelableEvent} on the instance of event that supports it.
 *
 * @param event Event that supports cancellation.
 */
declare function implementCancelation(event: CancelableEvent): void;
/**
 * @param event Event to check.
 * @returns Whether the event is cancellable.
 */
declare function isCancelable(event: Event): event is Event & CancelableEvent;
/**
 * Checks whether the provided event has been cancelled.
 *
 * There are two reasons you may prefer this utility function to using
 *
 * - {@link CancelableEvent.canceled} method is marked as configurable and can be
 *   overridden by bad code.
 * - You don't know the event type.
 *
 * @param event Event, which cancellation status needs a check.
 * @returns Always `false` if event is not cancellable, otherwise whether the
 *   event was cancelled or not.
 */
declare function isCanceled(event: Event): boolean;

/**
 * An event that is fired whenever the {@link IntlController} locale changes.
 *
 * This event is cancellable, cancelling it will result in locale staying the
 * same as it was.
 *
 * This event is called before {@link LocaleLoadEvent}.
 */
declare class LocaleChangeEvent implements Event {
    readonly previousLocale: string | null;
    readonly locale: string;
    readonly automatic: boolean;
    readonly type = "localechange";
    /**
     * @param previousLocale Previously used locale.
     * @param locale Locale that locale will be changed to.
     * @param automatic Whether the locale change was invoked by automation.
     */
    constructor(previousLocale: string | null, locale: string, automatic: boolean);
}
interface LocaleChangeEvent extends CancelableEvent {
}

/**
 * An event that is fired whenever the {@link IntlController} does not have a
 * locale loaded and requests it to be loaded. This event is called
 * asynchronously and can be used to implement custom message loaders.
 *
 * This event is cancellable, cancelling it will result in {@link IntlController}
 * throwing an error to the requestor of locale change.
 */
declare class LocaleLoadEvent implements Event {
    readonly type = "localeload";
    /** Locale descriptor for the locale that is being loaded. */
    readonly locale: LocaleDescriptor;
    /**
     * Intermediate object presenting messages that were loaded previously and
     * throughout this event's propagation to all listeners. It may or may not be
     * the actual messages map, depending on whether it was initialized before.
     */
    readonly messages: Partial<MessagesMap>;
    /**
     * Intermediate object presenting locale resources that were loaded previously
     * and throughout this event's propagation to all listener. It may or may not
     * be the actual resources map, depending on whether it was initialized
     * before.
     */
    readonly resources: Partial<VueIntlController.LocaleResources>;
    /**
     * @param localeDescriptor Locale descriptor that describes active locale.
     * @param locale Locale object which the initial values will be mapped from.
     */
    constructor(localeDescriptor: LocaleDescriptor, locale: Locale);
    /**
     * Adds provided messages to the the intermediate messages map, overriding any
     * existing values.
     *
     * For better type interference this method uses partial object notation
     * rather than parameters.
     *
     * @param messages Messages to add to the intermediate messages map.
     */
    addMessages(messages: Partial<MessagesMap>): void;
    /**
     * Adds provided resources to the the intermediate resources map, overriding
     * any existing values.
     *
     * For better type interference this method uses partial object notation
     * rather than parameters.
     *
     * @param resources Resources to add to the intermediate resources map.
     */
    addResources(resources: Partial<VueIntlController.LocaleResources>): void;
}
interface LocaleLoadEvent extends AsyncEvent, CancelableEvent, CollectableEvent<{
    messages: Partial<MessagesMap>;
    resources: Partial<VueIntlController.LocaleResources>;
}> {
}

declare class AutomaticStateChangeEvent implements Event {
    /** New state of automatic locale mode. */
    readonly state: boolean;
    readonly type = "automatic";
    constructor(
    /** New state of automatic locale mode. */
    state: boolean);
}
interface AutomaticStateChangeEvent extends CancelableEvent {
}

/**
 * An event which is fired whenever an error occurs within the IntlController or
 * associated listeners.
 */
declare class ErrorEvent implements Event {
    /** Error that occurred. */
    readonly error: unknown;
    readonly type = "error";
    constructor(
    /** Error that occurred. */
    error: unknown);
}

export { AsyncEvent as A, CollectableEvent as C, Event as E, LocaleChangeEvent as L, EventListenerOptions as a, EventTypes as b, EventObjectFrom as c, EventTarget as d, implementCollection as e, CancelableEvent as f, implementCancelation as g, isCancelable as h, isAsyncEvent as i, isCanceled as j, LocaleLoadEvent as k, AutomaticStateChangeEvent as l, markAsAsync as m, ErrorEvent as n };
