import { d as MessagesMap } from './messages-4c34bad5.js';
import { L as LocaleDescriptor, a as Locale } from './locales-8cef3d6b.js';
import { P as PreferredLocalesSource } from './sources-02c94682.js';
import { d as EventTarget, L as LocaleChangeEvent, k as LocaleLoadEvent, n as ErrorEvent, l as AutomaticStateChangeEvent, E as Event, a as EventListenerOptions } from './ErrorEvent-032e1b23.js';
import { Ref, ComputedRef } from 'vue';
import { I as IntlPartial } from './intl-0143b193.js';
import { AsyncComputedRef } from '@braw/async-computed';

interface AutomationPartial {
    /** BCP 47 code of the locale preferred by one of the automatic sources. */
    get preferredLocale(): string;
    /**
     * Adds a preferred locale source.
     *
     * If source is already added, it is removed and added at the new position,
     * that depends on the {@link prepend} argument.
     *
     * @param source Automatic locale source to add.
     * @param prepend Whether to prepend locale source before all other sources.
     *   Default is `false`.
     */
    addSource(source: PreferredLocalesSource, prepend?: boolean): void;
    /**
     * Removes a preferred locale source.
     *
     * @param source Locale source to remove.
     */
    removeSource(source: PreferredLocalesSource): void;
}

interface LocalesPartial {
    /** A reference to a map containing locale data. */
    get $locales(): Ref<Map<LocaleDescriptor, Locale>>;
    /** Read-only reference to the current loading promise. */
    get $loading(): AsyncComputedRef<void>;
    /**
     * Whether the automation is enabled and locale is picked automatically based
     * on one of the preferred locale sources.
     */
    get automatic(): boolean;
    /** @returns Whether the current locale is loaded. */
    get ready(): boolean;
    /** BCP 47 code of the currently used locale. */
    get locale(): string;
    /**
     * Finds a descriptor for one of the existing locales. Descriptor objects may
     * contain useful information regarding the presentation of locale.
     *
     * @param localeCode BCP 47 locale code to search for descriptor.
     * @returns Descriptor for the locale or `undefined`, if none exists.
     */
    getLocaleDescriptor(localeCode: string): LocaleDescriptor | undefined;
    /**
     * Adds the provided descriptor to the configuration.
     *
     * If locale for the provided locale code or descriptor already exists, that
     * locale will be replaced, causing all associated with it data to be
     * deleted.
     *
     * @param descriptor Descriptor or a BCP 47 locale code for which the
     *   descriptor is created.
     * @param force Whether to override any existing locale.
     * @returns Descriptor that was added.
     * @throws Error if locale with the same locale code exists but `force` is not
     *   provided or is set to `false`.
     */
    addLocale(descriptor: string | LocaleDescriptor, force?: boolean): LocaleDescriptor;
    /**
     * Removes the locale from the configuration.
     *
     * @param descriptor Descriptor or BCP 47 locale code of any existing locale.
     * @returns Removed locale descriptor or `null` if no locale was removed.
     */
    removeLocale(descriptor: string | LocaleDescriptor): LocaleDescriptor | null;
    /**
     * Adds messages for the locale, overwriting any existing ones, to the
     * internal messages map.
     *
     * @param descriptor Descriptor or a BCP 47 locale for which the messages are
     *   defined.
     * @param messages A map of messages for the locale for adding. This is
     *   partial, therefore any message keys that were assigned to `undefined`
     *   value will be removed.
     * @throws Error if locale with such code or descriptor does not exist.
     */
    addMessages(descriptor: string | LocaleDescriptor, messages: Partial<MessagesMap>): void;
    /**
     * Changes the current locale to one of defined locales based on the provided
     * code, or switches to one of the preferred locales and switches automatic
     * one, if `'auto'` is passed as a locale code.
     *
     * If passed locale code is not `'auto'`, proceeds to fire an event about the
     * locale change. If that event is cancelled, throws an exception. proceeds by
     * firing another event about locale load, any defined event handlers will
     * proceed to define locale messages in asynchronous manner, if any of them
     * throws an exception, than the whole.
     *
     * @param localeCode BCP 47 code of the locale to use.
     * @returns Promise that will resolve when the locale is loaded.
     */
    changeLocale(localeCode: 'auto' | (string & Record<never, never>)): Promise<void>;
    /**
     * Waits until the locale loading is complete and controller is ready for use.
     *
     * @throws If error occurs during the locale loading.
     */
    waitUntilReady(): Promise<void>;
}

interface LocaleDataPartial {
    /** Read-only reference to default messages map. */
    get $defaultMessages(): ComputedRef<MessagesMap>;
    /** Read-only reference to messages map for the current locale. */
    get $messages(): ComputedRef<Partial<MessagesMap>>;
    /** Read-only reference to the default resources map. */
    get $defaultResources(): ComputedRef<VueIntlController.LocaleResources>;
    /** Read-only reference to the active */
    get $resources(): ComputedRef<Partial<VueIntlController.LocaleResources>>;
    /** Read-only map of default messages. */
    get defaultMessages(): MessagesMap;
    /** Read-only map of messages for the current locale. */
    get messages(): Partial<MessagesMap>;
    /** Read-only resource map for the default locale. */
    get defaultResources(): VueIntlController.LocaleResources;
    get resources(): Partial<VueIntlController.LocaleResources>;
}

interface IntlController<ControllerType> extends AutomationPartial, ConfigPartial<ControllerType>, LocaleDataPartial, EventTargetPartial<ControllerType>, IntlPartial<ControllerType>, LocalesPartial {
}

type ControllerEvents = LocaleChangeEvent | LocaleLoadEvent | ErrorEvent | AutomaticStateChangeEvent;
type EventTargetPartial<T> = EventTarget<ControllerEvents, IntlController<T>>;

type ListenerItemRegular<ControllerType, EventType> = Extract<ControllerEvents, {
    type: EventType;
}> extends never ? never : (this: IntlController<ControllerType>, event: Extract<ControllerEvents, {
    type: EventType;
}>) => any;
type ListenerItemWithOptions<ControllerType, EventType> = {
    listener: ListenerItemRegular<ControllerType, EventType>;
    options?: EventListenerOptions;
};
type ListenerItem<ControllerType, EventType> = ListenerItemWithOptions<ControllerType, EventType> | ListenerItemRegular<ControllerType, EventType>;
type ControllerEventsMap<ControllerType> = {
    [EventType in ControllerEvents extends Event ? ControllerEvents['type'] : never]?: ControllerEvents extends Event ? EventType extends ControllerEvents['type'] ? ListenerItem<ControllerType, EventType> | ListenerItem<ControllerType, EventType>[] : never : never;
};

/** Represents the controller configuration (state). */
interface ControllerConfiguration<ControllerType> {
    /**
     * BCP 47 language tag of the currently used locale.
     *
     * @default 'en-US'
     */
    locale: string;
    /**
     * BCP 47 language tag of the default locale.
     *
     * @default 'en-US'
     */
    defaultLocale: string;
    /**
     * All the locale descriptors.
     *
     * Defaults to newly meta-less generated descriptors for the default and
     * current locales.
     *
     * @default [{ code: 'en-US' }]
     */
    locales: LocaleDescriptor[];
    /**
     * Whether the automatic language selection is enabled.
     *
     * @default false // Uses configured locale.
     */
    usePreferredLocale: boolean;
    /**
     * An array of preferred locale sources.
     *
     * The order in which sources defines the preference for one particular
     * source.
     *
     * @default [ ] // Fall backs to default locale.
     */
    preferredLocaleSources: PreferredLocalesSource[];
    /** Defines a map of event listeners to auto-bind. */
    listen: ControllerEventsMap<ControllerType>;
}
interface ConfigPartial<ControllerType> {
    /** Reactive configuration of the controller. */
    get $config(): ControllerConfiguration<ControllerType>;
    /** BCP 47 code of the default locale. */
    get defaultLocale(): string;
    /** {@link Intl.Locale} for the current locale. */
    get intlLocale(): Intl.Locale;
}

export { ControllerConfiguration as C, IntlController as I };
