'use strict';

const AsyncEventSymbol = Symbol('isAsync');
function markAsAsync(event) {
  Object.defineProperty(event, AsyncEventSymbol, {
    configurable: true,
    value: true
  });
}

/**
 * Checks whether the event is async event.
 *
 * @param event Event to check.
 * @returns Whether the event must be executed in async manner.
 */
function isAsyncEvent(event) {
  return AsyncEventSymbol in event && AsyncEventSymbol in event;
}

const CollectableSymbol = Symbol('isCollectable');
const CollectedPropertyKey = 'collected';
const CollectPropertyKey = 'collect';

/**
 * Represents a collectable event that returns {@link T} on collection.
 *
 * @template T The result of collection.
 */

function implementCollection(event, collector) {
  let collected = false;
  function wrappedCollector() {
    if (collected) {
      throw new Error('Cannot call collector for a second time');
    }
    try {
      return collector();
    } finally {
      collected = true;
    }
  }
  Object.defineProperties(event, {
    [CollectableSymbol]: {
      configurable: true,
      value: true
    },
    [CollectedPropertyKey]: {
      configurable: true,
      get() {
        return collected;
      }
    },
    [CollectPropertyKey]: {
      configurable: true,
      value: wrappedCollector
    }
  });
}

const Canceled = Symbol('isCancellable');
const CanceledPropertyKey = 'canceled';
const CancelPropertyKey = 'cancel';
/**
 * Implements {@link CancelableEvent} on the instance of event that supports it.
 *
 * @param event Event that supports cancellation.
 */
function implementCancelation(event) {
  let cancelled = false;
  Object.defineProperties(event, {
    [Canceled]: {
      configurable: true,
      get() {
        return cancelled;
      }
    },
    [CanceledPropertyKey]: {
      configurable: true,
      get() {
        return cancelled;
      }
    },
    [CancelPropertyKey]: {
      configurable: true,
      value: function cancel() {
        cancelled = true;
      }
    }
  });
}

/**
 * @param event Event to check.
 * @returns Whether the event is cancellable.
 */
function isCancelable(event) {
  return Canceled in event;
}

/**
 * Checks whether the provided event has been cancelled.
 *
 * There are two reasons you may prefer this utility function to using
 *
 * - {@link CancelableEvent.canceled} method is marked as configurable and can be
 *   overridden by bad code.
 * - You don't know the event type.
 *
 * @param event Event, which cancellation status needs a check.
 * @returns Always `false` if event is not cancellable, otherwise whether the
 *   event was cancelled or not.
 */
function isCanceled(event) {
  return isCancelable(event) && event[Canceled];
}

function _defineProperty(obj, key, value) {
  key = _toPropertyKey(key);
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }
  return obj;
}
function _toPrimitive(input, hint) {
  if (typeof input !== "object" || input === null) return input;
  var prim = input[Symbol.toPrimitive];
  if (prim !== undefined) {
    var res = prim.call(input, hint || "default");
    if (typeof res !== "object") return res;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return (hint === "string" ? String : Number)(input);
}
function _toPropertyKey(arg) {
  var key = _toPrimitive(arg, "string");
  return typeof key === "symbol" ? key : String(key);
}

/**
 * An event that is fired whenever the {@link IntlController} locale changes.
 *
 * This event is cancellable, cancelling it will result in locale staying the
 * same as it was.
 *
 * This event is called before {@link LocaleLoadEvent}.
 */
class LocaleChangeEvent {
  /**
   * @param previousLocale Previously used locale.
   * @param locale Locale that locale will be changed to.
   * @param automatic Whether the locale change was invoked by automation.
   */
  constructor(previousLocale, locale, automatic) {
    _defineProperty(this, "type", 'localechange');
    this.previousLocale = previousLocale;
    this.locale = locale;
    this.automatic = automatic;
    implementCancelation(this);
  }
}

function createHashMap() {
  return Object.create(null);
}

/**
 * An event that is fired whenever the {@link IntlController} does not have a
 * locale loaded and requests it to be loaded. This event is called
 * asynchronously and can be used to implement custom message loaders.
 *
 * This event is cancellable, cancelling it will result in {@link IntlController}
 * throwing an error to the requestor of locale change.
 */
class LocaleLoadEvent {
  /**
   * @param localeDescriptor Locale descriptor that describes active locale.
   * @param locale Locale object which the initial values will be mapped from.
   */
  constructor(localeDescriptor, locale) {
    var _locale$messages, _locale$resources;
    _defineProperty(this, "type", 'localeload');
    this.locale = localeDescriptor;
    this.messages = (_locale$messages = locale.messages) !== null && _locale$messages !== void 0 ? _locale$messages : createHashMap();
    this.resources = (_locale$resources = locale.resources) !== null && _locale$resources !== void 0 ? _locale$resources : createHashMap();
    implementCancelation(this);
    implementCollection(this, () => {
      this.cancel();
      return {
        messages: this.messages,
        resources: this.resources
      };
    });
    markAsAsync(this);
  }

  /**
   * Adds provided messages to the the intermediate messages map, overriding any
   * existing values.
   *
   * For better type interference this method uses partial object notation
   * rather than parameters.
   *
   * @param messages Messages to add to the intermediate messages map.
   */
  addMessages(messages) {
    Object.assign(this.messages, messages);
  }

  /**
   * Adds provided resources to the the intermediate resources map, overriding
   * any existing values.
   *
   * For better type interference this method uses partial object notation
   * rather than parameters.
   *
   * @param resources Resources to add to the intermediate resources map.
   */
  addResources(resources) {
    Object.assign(this.resources, resources);
  }
}

class AutomaticStateChangeEvent {
  constructor( /** New state of automatic locale mode. */
  state) {
    _defineProperty(this, "type", 'automatic');
    this.state = state;
    implementCancelation(this);
  }
}

/**
 * An event which is fired whenever an error occurs within the IntlController or
 * associated listeners.
 */
class ErrorEvent {
  // eslint-disable-next-line n/handle-callback-err
  constructor( /** Error that occurred. */
  error) {
    _defineProperty(this, "type", 'error');
    this.error = error;
  }
}

exports.AutomaticStateChangeEvent = AutomaticStateChangeEvent;
exports.ErrorEvent = ErrorEvent;
exports.LocaleChangeEvent = LocaleChangeEvent;
exports.LocaleLoadEvent = LocaleLoadEvent;
exports.createHashMap = createHashMap;
exports.implementCancelation = implementCancelation;
exports.implementCollection = implementCollection;
exports.isAsyncEvent = isAsyncEvent;
exports.isCancelable = isCancelable;
exports.isCanceled = isCanceled;
exports.markAsAsync = markAsAsync;
